//parser.c

#include <avr/io.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <string.h>

#include "XBOB4EncInt.h"
#include "uart.h"
#include "spi.h"
#include "switch.h"
#include "parser.h"

extern int32_t count;
extern uint8_t metric;
extern uint8_t screen;
extern uint8_t poscmd[9];
extern uint8_t clrcmd[5];
extern uint8_t mask;
extern uint8_t labelf;
extern uint8_t labelm;
extern uint16_t scaler;
extern uint8_t width;
extern uint8_t mode0x;
extern uint8_t mode0y;
extern uint8_t mode1x;
extern uint8_t mode1y;
extern uint8_t mode2x;
extern uint8_t mode2y;
extern uint8_t mode3x;
extern uint8_t mode3y;
extern uint8_t mode4x;
extern uint8_t mode4y;
extern double scalefactor;
uint8_t decimalstr[16];

void ParseCommand(void)
{
	static uint8_t strindex;
	static uint8_t state = PARSER_TOP;
	uint8_t inchar = GetChar();
	static uint16_t param1,param2,param3;
	switch(state)
	{
	//PARSER_TOP is the normal inactive wait state looking for <ESC> character
	case PARSER_TOP:
		param1 = 0;
		param2 = 0;
		param3 = 0;
		if (inchar == ESC)
		{
			state = GOT_ESC;
		}
		break;
	//<ESC> has been recieved looking for '[' any other character returns to PARSER_TOP 
	case GOT_ESC:
		if (inchar == '[')
		{
			state = GOT_CSI;
		}
		else
		{
			state = PARSER_TOP;
		}
		break;
	//<ESC> and '[' have been recieved completing the CSI
	//recive first parameter or commands 
	case GOT_CSI:
		//numeric values are a parameter
		if ((inchar >= '0')&&(inchar <= '9'))
		{
			param1 *= 10;
			param1 += inchar - '0';
		}
		//';' ends param1 proceeds to next state
		else if (inchar == ';')
		{
			state = GOT_PAR1;
		}
		//nonnumeric character indicates a command so reset state machine and 
		//proceed to the command interpreter
		else if((inchar >= '@') && (inchar <= '~'))
		{
			state = PARSER_TOP;
			ExecuteCommand(inchar,param1,param2,param3);
		}
		// anything else is invalid so reset to PARSER_TOP
		else
		{
			state = PARSER_TOP;
		}
		break;
	case GOT_PAR1:
		if ((inchar >= '0')&&(inchar <= '9'))
		{
			param2 *= 10;
			param2 += inchar - '0';
		}
		//special case for param2 floating point value
		else if (inchar == '.')
		{
			state = DECIMAL_PARAMETER;
			decimalstr[0] = '.';
			strindex = 1;
		}
		else if (inchar == ';')
		{
			state = GOT_PAR2;
		}
		else if((inchar >= '@') && (inchar <= '~'))
		{
			state = PARSER_TOP;
			ExecuteCommand(inchar,param1,param2,param3);
		}
		else
		{
			state = PARSER_TOP;
		}
		break;
	case GOT_PAR2:
		if ((inchar >= '0')&&(inchar <= '9'))
		{
			param3 *= 10;
			param3 += inchar - '0';
		}
		else if((inchar >= '@') && (inchar <= '~'))
		{
			state = PARSER_TOP;
			ExecuteCommand(inchar,param1,param2,param3);
		}
		else
		{
			state = PARSER_TOP;
		}
		break;
	// in order to recieve a floating point value input goes to a string to be 	
	//interpreted once the whole value is recieved
	case DECIMAL_PARAMETER:
		if ((strindex < 16)&&(inchar >= '0')&&(inchar <= '9'))
		{
			decimalstr[strindex] = inchar;
			strindex++;			
		}
		else if((inchar >= '@') && (inchar <= '~'))
		{
			state = PARSER_TOP;
			ExecuteCommand(inchar,param1,param2,param3);
		}
		else
		{
			state = PARSER_TOP;
		}
		
		break;
		
	}
}

void ExecuteCommand(uint8_t command,uint16_t param1,uint16_t param2,uint16_t param3)
{
	uint8_t *stopstr;
	uint8_t fontcmd[9];
	fontcmd[0] = 0x1b;
	fontcmd[1] = '[';
	fontcmd[2] = '0';
	fontcmd[3] = 'z';
	fontcmd[4] = 0x1b;
	fontcmd[5] = '[';
	fontcmd[6] = '1';
	fontcmd[7] = 'v';
	fontcmd[8] = 0;

	uint8_t configcmd[12];
	configcmd[0] = 0x1b;
	configcmd[1] = '[';
	configcmd[2] = '1';
	configcmd[3] = '6';
	configcmd[4] = ';';
	configcmd[5] = '0';
	configcmd[6] = 'v';
	configcmd[7] = 0x1b;
	configcmd[8] = '[';
	configcmd[9] = '1';
	configcmd[10] = 'v';
	configcmd[11] = 0;

	uint8_t statusstr[16];

	switch(command)
	{
	case 'c'://set calibration multiplier
	//if param1 = 0 then param2 is the scale multiplier 
	//if param1 = 1 then param2 is the desired display value (in feet)for
	//the current encoder count  
		if (param1 == 0)
		{
			scalefactor = param2 + strtod(decimalstr,&stopstr);
			eeprom_write_block (&scalefactor, (uint8_t *)EESCALEFACTOR, 4);
		}
		else if (param1 == 1)
		{
			scalefactor = ((double)(param2 + strtod(decimalstr,&stopstr))) / (double)(count/4);
			eeprom_write_block (&scalefactor, (uint8_t *)EESCALEFACTOR, 4);
		}
		memset(decimalstr,0,sizeof(decimalstr));
		SpiSendString(clrcmd);
		Refresh();
		break;

	case 'd'://set decimal point
	//sets the number of digits to the right of the decimal point
		if ((param1 >= 0) & (param1 <= 4)) 
		{
			mask = param1;
			eeprom_write_byte((uint8_t *)EEMASK,mask);
			SpiSendString(clrcmd);
			Refresh();
		}
		break;

	case 'f'://set font
		if (param1 == 0)//small font
		{
			screen &= ~2;
			fontcmd[2] = '0';
		}
		else if (param1 == 1)//large font
		{
			screen |= 2;
			fontcmd[2] = '5';
		}
		//save to eeprom
		eeprom_write_byte((uint8_t *)EESCREEN,screen);
		//tell BOB-4 that we changed the font
		SpiSendString(fontcmd);
		//set up new position command
		SpiSendString(clrcmd);
		Refresh();
		break;

	case 'p'://set display locations
		switch(param1)
		{
		case 0:
			mode0x = param2;
			mode0y = param3;
			eeprom_write_byte((uint8_t *)EEMODE0X,mode0x);
			eeprom_write_byte((uint8_t *)EEMODE0Y,mode0y);
			break;
		case 1:
			mode1x = param2;
			mode1y = param3;
			eeprom_write_byte((uint8_t *)EEMODE1X,mode1x);
			eeprom_write_byte((uint8_t *)EEMODE1Y,mode1y);
			break;
		case 2:
			mode2x = param2;
			mode2y = param3;
			eeprom_write_byte((uint8_t *)EEMODE2X,mode2x);
			eeprom_write_byte((uint8_t *)EEMODE2Y,mode2y);
			break;
		case 3:
			mode3x = param2;
			mode3y = param3;
			eeprom_write_byte((uint8_t *)EEMODE3X,mode3x);
			eeprom_write_byte((uint8_t *)EEMODE3Y,mode3y);
			break;
		case 4:
			mode4x = param2;
			mode4y = param3;
			eeprom_write_byte((uint8_t *)EEMODE4X,mode4x);
			eeprom_write_byte((uint8_t *)EEMODE4Y,mode4y);
			break;
		}
		SetPositionCommand();
		SpiSendString(clrcmd);
		Refresh();
		break;

	case 'r'://restore defaults
		//mode 0 upper left location
		mode = 0;
		eeprom_write_byte((uint8_t *)EEMODE,mode);

		//not metric
		metric = 0;
		eeprom_write_byte((uint8_t *)EEUNIT,metric);

		//small font/NTSC 
		screen = 0;
		fontcmd[2] = '0';
		SpiSendString(fontcmd);
		configcmd[5] = '0';
		SpiSendString(configcmd);
		eeprom_write_byte((uint8_t *)EESCREEN,screen);

		//1 decimal point
		mask = 1;
		eeprom_write_byte((uint8_t *)EEMASK,mask);

		//calibration scaler .10
		scalefactor = 0.1;
		eeprom_write_block (&scalefactor, (uint8_t *)EESCALEFACTOR, 4);

		//units labels
		labelf = 'F';
		eeprom_write_byte((uint8_t *)EELABELF,labelf);
		labelm = 'M';
		eeprom_write_byte((uint8_t *)EELABELM,labelm);

		//display width
		width = 7;
		eeprom_write_byte((uint8_t *)EEWIDTH,width);

		//default locations

		mode0x = 0;
		eeprom_write_byte((uint8_t *)EEMODE0X,mode0x);
		mode0y = 0;
		eeprom_write_byte((uint8_t *)EEMODE0Y,mode0y);

		mode1x = 26;
		eeprom_write_byte((uint8_t *)EEMODE1X,mode1x);
		mode1y = 0;
		eeprom_write_byte((uint8_t *)EEMODE1Y,mode1y);

		mode2x = 14;
		eeprom_write_byte((uint8_t *)EEMODE2X,mode2x);
		mode2y = 7;
		eeprom_write_byte((uint8_t *)EEMODE2Y,mode2y);

		mode3x = 0;
		eeprom_write_byte((uint8_t *)EEMODE3X,mode3x);
		mode3y = 15;
		eeprom_write_byte((uint8_t *)EEMODE3Y,mode3y);

		mode4x = 26;
		eeprom_write_byte((uint8_t *)EEMODE4X,mode4x);
		mode4y = 15;
		eeprom_write_byte((uint8_t *)EEMODE4Y,mode4y);

		SetPositionCommand();
		SpiSendString(clrcmd);
		Refresh();
		break;

	case 's'://get status report
		SendString(VERSION_STRING);
		PutChar(0x0d);

		SendString("Mode = ");
		PutChar(mode + '0');
		PutChar(0x0d);

		SendString("Font = ");
		PutChar(((screen & 2)>>1) + '0');
		PutChar(0x0d);
				
		SendString("Scale factor = ");
		dtostrf(scalefactor,8,5,statusstr);
		SendString(statusstr);
		PutChar(0x0d);

		SendString("Video Standard = ");
		PutChar((screen & 1) + '0');
		PutChar(0x0d);

		switch(mode)
		{
		case 0:
			itoa(mode0x,statusstr,10);
			SendString(statusstr);
			PutChar(',');
			itoa(mode0y,statusstr,10);
			SendString(statusstr);
			PutChar(0x0d);
			break;
		case 1:
			itoa(mode1x,statusstr,10);
			SendString(statusstr);
			PutChar(',');
			itoa(mode1y,statusstr,10);
			SendString(statusstr);
			PutChar(0x0d);
			break;
		case 2:
			itoa(mode2x,statusstr,10);
			SendString(statusstr);
			PutChar(',');
			itoa(mode2y,statusstr,10);
			SendString(statusstr);
			PutChar(0x0d);
			break;
		case 3:
			itoa(mode3x,statusstr,10);
			SendString(statusstr);
			PutChar(',');
			itoa(mode3y,statusstr,10);
			SendString(statusstr);
			PutChar(0x0d);
			break;
		case 4:
			itoa(mode4x,statusstr,10);
			SendString(statusstr);
			PutChar(',');
			itoa(mode4y,statusstr,10);
			SendString(statusstr);
			PutChar(0x0d);
			break;
	}
	break;

	case 'u'://set units label
	//if param1 = 0 set label for feet, if param1 = 1 set label for metric display
	//label is sent as decimal ascii value of character desired
		if (param1 == 0)
		{
			labelf = param2;
			eeprom_write_byte((uint8_t *)EELABELF,labelf);
		}
		else if (param1 == 1)
		{
			labelm = param2;
			eeprom_write_byte((uint8_t *)EELABELM,labelm);
		}
		SpiSendString(clrcmd);
		Refresh();	
		break;

	case 'v'://set video standard
		if (param1 == 0)//ntsc
		{
			screen &= ~1;
			configcmd[5] = '0';
		}
		else if (param1 == 1)//pal
		{
			screen |= 1;
			configcmd[5] = '1';
		}
		//save to eeprom
		eeprom_write_byte((uint8_t *)EESCREEN,screen);
		//tell BOB-4 to switch video standard
		SpiSendString(configcmd);
		//set up new position command
		SetPositionCommand();
		SpiSendString(clrcmd);
		Refresh();
		break;
	case 'w'://set display width
		if ((param1 > 1) && (param1 < 17))
		{
			width = param1;
			//save to eeprom
			eeprom_write_byte((uint8_t *)EEWIDTH,width);
			SpiSendString(clrcmd);
			Refresh();
		}
		break;
	}
}
