//switch.c

#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <stdlib.h>

#include "XBOB4EncInt.h"
#include "switch.h"
#include "uart.h"
#include "spi.h"

extern uint8_t metric;
extern uint8_t screen;
extern int32_t count;
extern void CountToAscii(double);
extern uint8_t poscmd[];
uint8_t eventcount;
extern uint8_t mode0x;
extern uint8_t mode0y;
extern uint8_t mode1x;
extern uint8_t mode1y;
extern uint8_t mode2x;
extern uint8_t mode2y;
extern uint8_t mode3x;
extern uint8_t mode3y;
extern uint8_t mode4x;
extern uint8_t mode4y;
extern double scalefactor;

void get_count(void);
void set_count(void);
/**********************************************/

void SwitchInit(void)
{
	//all 5 pins set to input
	SWITCHDDR &= (~(1<<MODEPIN) & ~(1<<UNITPIN) & ~(1<<INCPIN) & ~(1<<DECPIN) & ~(1<<ZEROPIN));
	//turn on internal pullups
	SWITCHOUTPORT |= ((1<<MODEPIN) | (1<<UNITPIN) | (1<<INCPIN) | (1<<DECPIN) | (1<<ZEROPIN));
	//set up debounce timer,ctc mode,clk/256
	TCCR1B |= (1<<CS12) | (1<<WGM12);

	OCR1A = DEBOUNCE_PERIOD;
	TIFR1 |= (1<<OCF1A);   // Clear compare match flag.
	TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.
	debounce = 1;
	eventcount = 0;
}
/**********************************************/
//switch debounce timer
ISR(TIMER1_COMPA_vect)
{
	debounce--;
	if (!debounce)
	{
		TIMSK1 &= ~((1<<OCIE1A));
	}
}

/**********************************************/
void ReadSwitches(void)
{
	extern uint8_t clrcmd[];
	//read value of the switches
	//0 = switch pressed
	uint8_t val = (SWITCHINPORT & ((1<<MODEPIN)|(1<<UNITPIN)|(1<<INCPIN)|(1<<DECPIN)|(1<<ZEROPIN)));
	//biwise inversion so 1 = switch pressed
	val = ~val;
	
	//if no switches are pressed, let's not waste time
	if (!(val & 0x1f))
	{
		eventcount = 0;
		return;
	}
	//cycle through display modes
	if ((val & (1<<MODEPIN)))
	{
		debounce = NORMAL_DEBOUNCE;
		TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.
		mode++;
		if (mode > 5)
			mode = 0;
		SetPositionCommand();

		//refresh the display
		SpiSendString(clrcmd);
		if (mode !=5)
		{
			Refresh();
		}
		//save mode to EEPROM
		eeprom_write_byte((uint8_t *)EEMODE,mode);
	}
	//toggle feet/meters
	else if ((val & (1<<UNITPIN)))
	{
		debounce = NORMAL_DEBOUNCE;
		TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.
		if (metric)
		{
			metric = FALSE;
		}
		else
		{
			metric = TRUE;
		}
		//refresh the display
		SpiSendString(clrcmd);
		if (mode !=5)
		{
			Refresh();
		}
		eeprom_write_byte((uint8_t *)EEUNIT,metric);
	}
	else if ((val & (1<<INCPIN)) && (val & (1<<DECPIN)))
	{
		debounce = NORMAL_DEBOUNCE;
		TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.
		count = 0;
		set_count();
	}
	//zero the display
	else if (val & (1<<ZEROPIN))
	{
		debounce = NORMAL_DEBOUNCE;
		TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.
		count = 0;
		set_count();
	}
	//increment the count
	else if (val & (1<<INCPIN))
	{
		if (eventcount < 10)
			debounce = NORMAL_DEBOUNCE;
		else
			debounce = SHORT_DEBOUNCE;

		TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.

		if (eventcount < 11)
			eventcount++;
		get_count();
		count++;
		set_count();
	}
	//decrement the count
	else if (val & (1<<DECPIN))
	{
		if (eventcount < 10)
			debounce = NORMAL_DEBOUNCE;
		else
			debounce = SHORT_DEBOUNCE;

		TIMSK1 |= (1<<OCIE1A); // Enable compare match interrupt.
		
		if (eventcount < 11)
			eventcount++;
		
		get_count();
		count--;
		set_count();
	}
}

/**********************************************/
void SetPositionCommand(void)
{
	//insert x and y values for the current display mode into the position command string
	uint8_t positionstr[3];
	switch(mode)
	{
	//top left
	case 0:
		itoa(mode0x,positionstr,10);

		if(mode0x < 10)
		{
			poscmd[5] = '0';
			poscmd[6] = positionstr[0];
		}
		else
		{
			poscmd[5] = positionstr[0];
			poscmd[6] = positionstr[1];
		}

		itoa(mode0y,positionstr,10);
		if(mode0y < 10)
		{
			poscmd[2] = '0';
			poscmd[3] = positionstr[0];
		}
		else
		{
			poscmd[2] = positionstr[0];
			poscmd[3] = positionstr[1];
		}
		break;
	//top right
	case 1:
		itoa(mode1x,positionstr,10);

		if(mode1x < 10)
		{
			poscmd[5] = '0';
			poscmd[6] = positionstr[0];
		}
		else
		{
			poscmd[5] = positionstr[0];
			poscmd[6] = positionstr[1];
		}

		itoa(mode1y,positionstr,10);
		if(mode1y < 10)
		{
			poscmd[2] = '0';
			poscmd[3] = positionstr[0];
		}
		else
		{
			poscmd[2] = positionstr[0];
			poscmd[3] = positionstr[1];
		}
		break;
	//bottom left
	case 2:
		itoa(mode2x,positionstr,10);

		if(mode2x < 10)
		{
			poscmd[5] = '0';
			poscmd[6] = positionstr[0];
		}
		else
		{
			poscmd[5] = positionstr[0];
			poscmd[6] = positionstr[1];
		}

		itoa(mode2y,positionstr,10);
		if(mode2y < 10)
		{
			poscmd[2] = '0';
			poscmd[3] = positionstr[0];
		}
		else
		{
			poscmd[2] = positionstr[0];
			poscmd[3] = positionstr[1];
		}
		break;
	case 3:
		itoa(mode3x,positionstr,10);

		if(mode3x < 10)
		{
			poscmd[5] = '0';
			poscmd[6] = positionstr[0];
		}
		else
		{
			poscmd[5] = positionstr[0];
			poscmd[6] = positionstr[1];
		}

		itoa(mode3y,positionstr,10);
		if(mode3y < 10)
		{
			poscmd[2] = '0';
			poscmd[3] = positionstr[0];
		}
		else
		{
			poscmd[2] = positionstr[0];
			poscmd[3] = positionstr[1];
		}
		break;
	//bottom right
	case 4:
		itoa(mode4x,positionstr,10);

		if(mode4x < 10)
		{
			poscmd[5] = '0';
			poscmd[6] = positionstr[0];
		}
		else
		{
			poscmd[5] = positionstr[0];
			poscmd[6] = positionstr[1];
		}

		itoa(mode4y,positionstr,10);
		if(mode4y < 10)
		{
			poscmd[2] = '0';
			poscmd[3] = positionstr[0];
		}
		else
		{
			poscmd[2] = positionstr[0];
			poscmd[3] = positionstr[1];
		}
		break;
	}
		//automatically adjust positions depending on font and video standard
	/*	switch(mode)
		{
		//top left
		case 0:
			poscmd[2] = '0';
			poscmd[3] = '0';
			poscmd[5] = '0';
			poscmd[6] = '0';
			break;
		//top right
		case 1:
			poscmd[2] = '0';
			poscmd[3] = '0';
			if((screen == SCRN_SN)|(screen == SCRN_SP))
			{
				poscmd[5] = '2';
				poscmd[6] = '6';
			}
			else
			{
				poscmd[5] = '2';
				poscmd[6] = '4';
			}
			break;
		//center
		case 2:
			if (screen == SCRN_SP)
			{
				poscmd[2] = '0';
				poscmd[3] = '9';
				poscmd[5] = '1';
				poscmd[6] = '4';
			}
			else if (screen == SCRN_SN)
			{
				poscmd[2] = '0';
				poscmd[3] = '7';
				poscmd[5] = '1';
				poscmd[6] = '4';
			}
			else if ((screen == SCRN_LN)|(screen == SCRN_LP))
			{
				poscmd[2] = '0';
				poscmd[3] = '3';
				poscmd[5] = '1';
				poscmd[6] = '2';
			}
			break;
		//bottom left
		case 3:
			if (screen == SCRN_SP)
			{
				poscmd[2] = '1';
				poscmd[3] = '8';
			}
			else if (screen == SCRN_SN)
			{
				poscmd[2] = '1';
				poscmd[3] = '5';
			}
			else if (screen == SCRN_LN)
			{
				poscmd[2] = '0';
				poscmd[3] = '5';
			}
			else if (screen == SCRN_LP)
			{
				poscmd[2] = '0';
				poscmd[3] = '6';
			}
			poscmd[5] = '0';
			poscmd[6] = '0';
			break;
		//bottom right
		case 4:
			if (screen == SCRN_SP)
			{
				poscmd[2] = '1';
				poscmd[3] = '8';
				poscmd[5] = '2';
				poscmd[6] = '6';
			}
			else if (screen == SCRN_SN)
			{
				poscmd[2] = '1';
				poscmd[3] = '5';
				poscmd[5] = '2';
				poscmd[6] = '6';
			}
			else if (screen == SCRN_LN)
			{
				poscmd[2] = '0';
				poscmd[3] = '5';
				poscmd[5] = '2';
				poscmd[6] = '4';
			}
			else if (screen == SCRN_LP)
			{
				poscmd[2] = '0';
				poscmd[3] = '6';
				poscmd[5] = '2';
				poscmd[6] = '4';
			}
			break;
		}*/
}
/**********************************************/
